// INIT script

/**
 * @brief : Initial class for IceWarp WebMail
 * @date  : 6.3.2006 19:01:49
 * @status: draft
 * @require: storage
 *

  ? check opener (for valid webmail window)
  - parse input vars to GLOBAL cookie_vars and url_vars
  - choose page ($page <- "login" <- cookie_vars <- url_vars)
  - check ID (!isset than page = login)
  - load settings (if doesnt exist in [opener.]dataset)
  - get Lang (depeds on settings)
*/

function cInit(){

	//registr function to global variable, must be here because it will be used
	//window.oWM_INIT = this;
	
	// hide copyright box
	var eCopyright = document.getElementById('wm_copyright');
	if (eCopyright){
		eCopyright.style.display = 'none';
		eCopyright = null;
	}

	// allowed content
	var allowed_cookie = {"sid":'',"frm":''},
		allowed_get    = {"page":'index',"debug":null,"frm":null,"sid":null,"ref":null,"user":null,"pass":null,"mailto":null,"video":null};

	// name of main webmail cookie
	this.wm_cookie = 'wm_cookie';

	// parse cookies and get vars
	this._vars = arrConcat(arrKeySlice(cookieManager.get(this.wm_cookie), allowed_cookie), arrKeySlice(parseURL(), allowed_get));

	// DEBUG
	//if (this._vars['debug']) window.api_debug = new cAPI_debug();

	storage.library('gw_others');
	GWOthers.load(['skins','login_settings','layout_settings','forgot_settings','restrictions','signup_domains']);

	// get Language
	storage.language(GWOthers.getItem('LAYOUT_SETTINGS','language'));

	// create connection object
	gui._create('connection','obj_connection');

	if (this._vars['sid']){
		var sid = auth.login({"session":this._vars['sid']});

		//Add SID into dataset
		if (sid)
			dataSet.add('main',['sid'],sid);
		else{
			if (Is.Object(auth.error)){

				inspect2(auth.error);
				
				switch (auth.error.id){
					case 'login_account_valid':
						alert(getLang('ERROR::ACCOUNT_DISABLED'));
						break;
					case 'login_invalid':
						alert(getLang('ERROR::INVALID_LOGIN'));
						break;
				}
			}
			else
				alert(getLang('ERROR::INVALID_LOGIN'));

			gui._create('frm_login','frm_login','','',1);
			return;
		}

		//Save referrer, WM will be redirected back to this adress after logout
		if (this._vars['ref'])
			dataSet.add('main',['referrer_url'],this._vars['ref']);
		else
		if (document.referrer && document.referrer != document.location.href)
			dataSet.add('main',['referrer_url'], document.referrer);

		//start login sequence
		this._checkBrowserVersion();
		return;
	}

	if (this._vars['mailto']){
		try{
            var sid = '';
			if (this._vars['sid']){
				sid = auth.login({"session":this._vars['sid']});
				
				alert(this._vars['sid']+"\n"+sid);
			}
			else
			if (this._vars['user'] && this._vars['pass'])
				sid = auth.login({"username":this._vars['user'],"password":this._vars['pass']});
			else{
                var data = cookieManager.get('LoginData'),
					state = cookieManager.get('LoginState');

                if ((state==2 || state==3) && data.user && data.pass)
					sid = auth.login({"username":data.user,"digest":data.pass});
			}
		}
		catch(r){
			alert(getLang('ERROR::CONNECTION')+(typeof r == 'string'?":\n"+r:''));
		}

		//Login was successful
		if (sid && !auth.error){

			//Hide copyright block
			var celm;
			if((celm = document.getElementById('gui.svn'))){
				celm.style.display = 'none';
				celm = null;
			}

			this._checkBrowserVersion();
			return;
		}
		else{
			if (Is.Object(auth.error)){
				switch (auth.error.id){
					case 'login_account_valid':
						alert(getLang('ERROR::ACCOUNT_DISABLED'));
						break;
					case 'login_invalid':
						alert(getLang('ERROR::INVALID_LOGIN'));
						break;
				}
			}
			else
				alert(getLang('ERROR::INVALID_LOGIN'));

			gui._create('frm_login','frm_login','','',1);
		}
	}
	else
	switch (this._vars['frm']){
	// MAIN page
	case 'main':
		auth.login({"username":this._vars['user'] || 'admin',"password":this._vars['pass'] || 'asd'});
		gui._create('frm_main','frm_main');
		break;

	// LOGIN page
	default:
		gui._create('frm_login','frm_login','','',1);
	}
};

/**
 * Login sequence
 * @note: moved from frm_login (4.7.2007 10:58:11)
 **/
cInit.prototype._checkBrowserVersion = function () {

	switch (currentBrowser()) {

		case 'Mozilla':
		case 'MSIE7':
		case 'Opera':
		case 'Safari':
			this._continueLogin(); // do nothing, the browser is supported
			break;

		case 'MSIE6':

			storage.library('wm_storage');
			storage.library('gw_others');

			if (GWOthers.getItem('RESTRICTIONS', 'disable_ie6warning')){
				this._continueLogin();
				break;
			}

			//nacist "resources" do datasetu "storage"
			dataSet.add('storage','',WMStorage.get({'resources':['login_data']}));

			//precist data z datasetu "storage"
			var iLastShow = parseInt(GWOthers.getItem('LOGIN_DATA', 'ie_6_warning_show_on'));

			var iNow = parseInt((new Date()).getTime());
			if (!Is.Number(iLastShow) || iLastShow <= iNow-(this._showIEWarningFrequency*86400000)){
				GWOthers.set('LOGIN_DATA',{'ie_6_warning_show_on':iNow},'storage');
				WMStorage.set({'resources':dataSet.get('storage')},'storage');

				var cfm = gui._create('frm_confirm','frm_confirm','','', [this, '_continueLogin'], 'CONFIRMATION::BROWSER_WARNING_TITLE','CONFIRMATION::BROWSER_WARNING_TEXT_IE6');
				    cfm.btn_cancel._onclick = function() {
						cfm._destruct();
						document.location.href = './basic';
					};
			}
			else
				this._continueLogin();

			break;

		default:
			if (window.confirm(getLang('CONFIRMATION::BROWSER_WARNING_TEXT_UNSUPPORTED')))
				this._continueLogin();
			else
				document.location.href = './basic';
				
			break;
	}
};

cInit.prototype._continueLogin = function (b) {

	var me = this;

	if (!gui.preloader)
		gui._create("preloader","obj_loader");

	// [Sync] load of BIG JavaSript
	setTimeout(function(){if (gui.preloader) gui.preloader._value(getLang('PRELOADER::FRAMEWORK'));},0);
	storage.library('javascript');

    var old_skin = GWOthers.getItem('LAYOUT_SETTINGS', 'skin');

	// [Sync] load resources
	setTimeout(function(){if (gui.preloader) gui.preloader._value(getLang('PRELOADER::SETTINGS'));},0);
	GWOthers.load([	'skins',
					'im',
					'sip',
					'mail_settings_default',
					'mail_settings_general',
					'layout_settings',
					'homepage_settings',
					'calendar_settings',
					'default_calendar_settings',
					'cookie_settings',
					'default_reminder_settings',
				//	'default_folders',
					'spellchecker_languages',
					'signature',
					'restrictions',
					'personalities',
					'read_confirmation',
					'global_settings',
					'paths']);

	// Clear already downloaded templates
	if (old_skin != GWOthers.getItem('LAYOUT_SETTINGS', 'skin'))
	    storage.aStorage.template = {};

	// [Sync] load language
	setTimeout(function(){if (gui.preloader) gui.preloader._value(getLang('PRELOADER::LANGUAGE'));},0);
	storage.language(GWOthers.getItem('LAYOUT_SETTINGS', 'language'));

	// get accounts
	var aAccounts = WMAccounts.list();
	dataSet.add('accounts','',aAccounts);

	// refresh main account & continue preloading...
	

    this.__refreshed = false;
	WMFolders.list({'aid':sPrimaryAccount},'folders',[sPrimaryAccount],[me,'_updatePreloader']);

	/*
	for (var sAccId in aAccounts)
		if (aAccounts[sAccId] && aAccounts[sAccId]['PRIMARY']){
			// call the refresh via timeout so that the preloader is shown properly
			setTimeout(
			function(){

				if (gui.preloader)
					gui.preloader._value(getLang('PRELOADER::INDEXING'));

				WMAccounts.refresh({'aid':sAccId},'folders',[sAccId],[me,'_updatePreloader']);
			},0);
			break;
		}
	*/
};

cInit.prototype._updatePreloader = function (sError) {

	if (!dataSet.get('folders',[sPrimaryAccount])){
		this.__refreshed = true;
        setTimeout(function(){if (gui.preloader) gui.preloader._value(getLang('PRELOADER::INDEXING'));},0);
		WMAccounts.refresh({'aid':sPrimaryAccount},'folders',[sPrimaryAccount],[this,'_updatePreloader']);
    	return;
	}

	// works only with WMAccounts
	if (typeof sError == 'string'){
		var state = cookieManager.get('LoginState');
		if (state === '3')
			cookieManager.set('LoginState','2',30);

		alert(sError);

		window.location.href = window.location.href;
		return;
	}

	// call the refresh via timeout so that the preloader is updated properly
	var me = this;
	setTimeout(function(){
		if (gui.preloader)
			gui.preloader._value(getLang('PRELOADER::STARTUP'));
			
		me._startMain();
	},0);
}

cInit.prototype._startMain = function () {
	// [Async] Load style.css (marged css files)
	storage.css('style');
	storage.template('templates');
	storage.preloadObj();

	if (gui.preloader)
		gui.preloader._destruct();

	gui._create("frm_main","frm_main","","",true);

	if (this._vars['mailto']){

		var head = {};
		if (this._vars['mailto'].indexOf('?')>-1){
			head = parseURL(this._vars['mailto']);
			head.to = unescape(this._vars['mailto'].substr(0,this._vars['mailto'].indexOf('?')));
		}
		else
			head.to = unescape(this._vars['mailto']);

		NewMessage.compose(head);
	}
	
/*
	if (!this.__refreshed)
		WMAccounts.refresh({'aid':sPrimaryAccount},'folders',[sPrimaryAccount]);
*/
};
///////////////////////////
function initgui(){
	window.oWM_INIT = new cInit();
};