/**
 * @brief  : This class provides storage for GUI data and takes care about JS and CSS loading into page
 * @require: httprequest, browser_ext
 * @status : final
 * @date   : 6.3.2006 13:59:51
 **/
function cStorage(){
	this.aStorage = {"css":{},"library":{},"language":{},"template":{},"object":{}};

	this.aStorage.library = {
	//preloaded in startscript.js
	'client/inc/debug':true,'client/inc/object_ext':true,'client/inc/browser_ext':true,'client/inc/template':true,'client/inc/xmltools':true,
	'client/inc/httprequest':true,'client/inc/dataset':true,'client/inc/storage':true,'client/inc/gui':true,'client/inc/wm_generic':true,
	'client/inc/wm_auth':true,'client/inc/wm_accounts':true,'client/inc/wm_folders':true,'client/inc/wm_items':true,'client/inc/wm_settings':true,'client/inc/wm_storage':true,'client/inc/init':true,
	'client/inc/gw_others':true,'client/inc/json':true,

	//obj from Login screen
	'client/inc/obj_container_generic':true,'client/inc/obj_form_generic':true,'client/inc/obj_form_restrict':true,'client/inc/obj_label':true,'client/inc/obj_tabs':true,'client/inc/obj_tab':true,'client/inc/obj_input':true,'client/inc/obj_button':true,
	'client/inc/obj_select':true,'client/inc/obj_loader':true,'client/inc/frm_login':true
	};

	this.request = new cHttpRequest();

//	this.request.bSync = true;
};

/**
 * @brief: Append CSS file into page or enabled previously appended one
 * @date: 6.3.2006 13:56:53
 */
cStorage.prototype.css = function(sName) {

	// MSIE workaround because 31 link tag limitation  (style.css contains all *.css)
	if (typeof this.aStorage.css['style'] != 'undefined') return true;

	//check cache
	if (typeof this.aStorage.css[sName] != 'undefined') {
		if (this.aStorage.css[sName] == 'disabled') {
			document.getElementById('css_' + sName).disabled = false;
			this.aStorage.css[sName] = 'enabled';
		}
		return true;
	}

	//append CSS into page
	var cssobj = mkElement('link',{"rel":'stylesheet',"type":'text/css',"href":'client/skins/'+GWOthers.getItem('LAYOUT_SETTINGS', 'skin')+'/css/'+sName+'.css'});
	cssobj.id = 'css_' + sName;
	document.getElementsByTagName('head')[0].appendChild(cssobj);
	cssobj = null;
	//save name to cache
	this.aStorage.css[sName]='enabled';

	return true;
};


/**
 * @brief: disable given css in document
 * (remove doesnt work in MSIE - needs some research here!)
 * @date: 6.3.2006 13:56:21
 */
cStorage.prototype.remove_css = function (sName) {
	if (typeof this.aStorage.css[sName] != 'undefined') {
		if (this.aStorage.css[sName] == 'enabled') {
			document.getElementById('css_' + sName).disabled = true;
			this.aStorage.css[sName] = 'disabled';
		}
		return true;
	}
	return false;
};

cStorage.prototype.remove_all_css = function (){
	var tmp;
	for (var i in this.aStorage.css){
		if ((tmp = document.getElementById('css_' + i))){
			tmp.disabled = true;
			tmp.parentNode.removeChild(tmp);
		}

		delete this.aStorage.css[i];
	}
};
/**/

/**
 * @brief: Append library into page
 * (little odd behaviour in Safari, "var" definitions aren't eval as public!)
 */
cStorage.prototype.library = function(sName,sPath,bASync) {

	var sFile;
	switch (sPath){
		case 'skin':
			sFile = 'client/skins/'+GWOthers.getItem('LAYOUT_SETTINGS', 'skin')+'/inc/'+sName;
			break;
		default:
			sFile = 'client/inc/'+sName;

			// JS preloader
			if (this.aStorage.library['client/inc/javascript']) return true;
	}

	//check cache
	if (this.aStorage.library[sFile]) return true;

    //retrieve Lib from server
	var ASrequest = new cHttpRequest();
        ASrequest.sURL = sFile + '.js';

	if (bASync)
		ASrequest.sendXML('',[this,'preloadLib',['library',sFile,sName],'Text']);
	else{
	    ASrequest.sendXML();
	    var str = ASrequest.responseText();

		if (str.length<1){
		    ASrequest = null;
			return false;
		}

		//append library into page
		pubEval(str);

		//save name to cache
	    this.aStorage.library[sFile]='enabled';
    }
    
	ASrequest = null;
	return true;
};

/**
 * Synchronous preload f selected objects in advance
 * (MUST be in synchro mode)
 **/
cStorage.prototype.preloadObj = function(){
	//retrieve object from server
	this.request.sURL = 'client/objects/objects.xml';
	this.request.sendXML();

	var aObjects = this.request.responseArray();
	for(var i in aObjects.OBJECTS[0])
        this.object(i.toLowerCase(),aObjects.OBJECTS[0][i][0]);
};

/**
 * Asynchronous preload of selected templates
 **/
cStorage.prototype.preloadTpl = function(aResponseData,sName){
	if (aResponseData && sName){
		if (sName == 'templates'){
			if (aResponseData.Array && aResponseData.Array.TEMPLATE && aResponseData.Array.TEMPLATE[0])
				for(var i in aResponseData.Array.TEMPLATE[0])
					this.aStorage.template[i.toLowerCase()] = aResponseData.Array.TEMPLATE[0][i][0].VALUE || '';
		}
		else
			this.aStorage.template[sName] = aResponseData.Text || '';

		return;
	}
};

/**
 * @brief:  load and reconstruct language array from server
 * @return: language array
 */
cStorage.prototype.language = function(sName){

	//check cache
	if (typeof this.aStorage.language["_ACTIVE_LANG"] != 'undefined' && this.aStorage.language["_ACTIVE_LANG"] == sName)
		return sName;

	//retrieve language from server
	if(!sName)
	    sName = 'en';

	try{
		this.request.sURL = 'client/languages/'+sName+'/data.xml';
		this.request.sendXML();
		var lang = this.request.responseArray();
	}
	catch(e){
		var lang = null;
	}
	
	if (!Is.Array(lang)) {
		// try to load default language
		this.request.sURL = 'client/languages/en/data.xml';
		this.request.sendXML();
		lang = this.request.responseArray();
		if (!Is.Array(lang))
			throw new Error("cStorage.language() - bad language file syntax: " + sName);
	}

	//construct storage
	lang = lang['LANGUAGE'][0];
	var tmp = {};
	for (var i in lang){
		this.aStorage.language[i] = {};
		for (var j in lang[i][0]){
			if (j == 'VALUE') continue;
			
			var value = lang[i][0][j][0]['VALUE'];
			this.aStorage.language[i][j] = (Is.Defined(value)) ? value : '';
		}
	}

	return this.aStorage.language["_ACTIVE_LANG"] = sName;
};


/**
 * @brief :  load and template file from sever
 * @return: template as string
 * @date  : 4.7.2006 12:37:13
 */
cStorage.prototype.template = function(sName,bASync) {
	//check cache
	if (typeof this.aStorage.template[sName]!='undefined')
		return this.aStorage.template[sName];

    var skin = GWOthers.getItem('LAYOUT_SETTINGS', 'skin');

	//retrieve skin from server
	if (bASync){
        var ASrequest = new cHttpRequest();
            ASrequest.sURL = 'client/skins/'+skin+'/templates/'+sName+(sName=='templates'?'.xml':'.tpl');
			ASrequest.sendXML('',[this,'preloadTpl',[sName],(sName=='templates'?'Array':'Text')]);
	}
	else
	if (sName == 'templates'){
		this.request.sURL = 'client/skins/'+skin+'/templates/templates.xml';
		this.request.sendXML();
		this.preloadTpl({Array:this.request.responseArray()},'templates');
	}
	else{
		this.request.sURL = 'client/skins/'+skin+'/templates/'+sName+'.tpl';
		this.request.sendXML();

		var str = this.request.responseText();
		if (!str)
			throw new Error("cStorage.template() - blank template file: " + skin+'/'+sName);

		//save to cache
		return this.aStorage.template[sName] = str;
	}
};

/**
 * @brief:  load  page descriptor from sever
 * @return: page as array
 */
cStorage.prototype.object = function(sName, aObject) {

	//check cache
	if (typeof this.aStorage.object[sName]!='undefined')
		return this.aStorage.object[sName];
		
	//retrieve object from server
	if (!aObject){
		this.request.sURL = 'client/objects/'+sName+'.xml';
		this.request.sendXML();
		aObject = this.request.responseArray();
	}

	//check for <object> root and remove it
	try{
		aObject = aObject['OBJECT'][0];
	}
	catch(e){
		throw "cStorage.object() - blank xml: " + sName;
	}

	//load CSS
	if (aObject['CSS'])
		for(var i in aObject['CSS'])
			this.css(aObject['CSS'][i]['VALUE']);

	//load JS
	if (aObject['LIBRARY'])
		for(var i in aObject['LIBRARY']){
            if (aObject['LIBRARY'][i]['ATTRIBUTES'] && aObject['LIBRARY'][i]['ATTRIBUTES']['PATH'])
            	this.library(aObject['LIBRARY'][i]['VALUE'],aObject['LIBRARY'][i]['ATTRIBUTES']['PATH']);
			else
				this.library(aObject['LIBRARY'][i]['VALUE']);
		}

	//save to cache
	return this.aStorage.object[sName] = aObject;
};

storage = new cStorage();

///////////////////////////

function getLang(str,aSubstitute,nobr){

	if (typeof str != 'string' || !str) return '';
    
	var out = '',
		a = str.toUpperCase().split('::');

	try{
		if (typeof a[1] == 'undefined')
			out = storage.aStorage.language[a[0]];
		else
			out = storage.aStorage.language[a[0]][a[1]];

		if (aSubstitute && typeof out == 'string'){
			var parts = out.split('%s');
			out = parts.shift();
			for (var i in parts)
				out += aSubstitute.shift() + parts[i];
		}

		return out;
	}
	catch(e){}

	return (nobr?str:'{'+str+'}');
};