/**
 * @brief: WebmailIqAuth handling class
 **/
function wm_auth(){
	this.xmlns='auth';
};

wm_auth.inherit(wm_generic);

var _me=wm_auth.prototype;

/**
 * param:
 *	auth_array[username]
 *	(auth_array[password] or auth_array[digest]) or auth_array[session]
 **/
_me.login = function(auth_array,sDataSet,sDataPath){

	if (auth_array['username'] && (auth_array['password'] || auth_array['digest'])){

		// encrypt password
		if (!auth_array.digest){
			try{
				auth_array.digest = this.digest(auth_array['username'],auth_array['password']);
			}
			catch(e){
				this.error = {id:'RSA'};
				return false;
			}
		}

		// Logout prevous user from dataset
		this.logout();

		// prepare data for request
		var request={"USERNAME":[{"VALUE":auth_array.username}],"DIGEST":[{"VALUE":auth_array.digest}],"METHOD":[{"VALUE":'RSA'}]};

		if (auth_array.disable_ip_check)
			request.DISABLE_IP_CHECK = [{"VALUE":1}];
	}
	else
	if (auth_array.session)
		// prepare data for request
		var request = {"SESSION":[{"VALUE":auth_array.session}]};
	else
		return false;
		
	/*
	if (!auth_array['username'] || (!auth_array['password'] && !auth_array['digest'])) return false;

	// encrypt password
	if (!auth_array.digest)
		auth_array.digest = this.digest(auth_array['username'],auth_array['password']);

	// Logout prevous user from dataset
	this.logout();

	// prepare data for request
	var request={"USERNAME":[{"VALUE":auth_array.username}],"DIGEST":[{"VALUE":auth_array.digest}],"METHOD":[{"VALUE":'RSA'}]};

	// cookie check
	/*
	if (document.cookie){
		var uid = unique_id();

		//document.cookie = "auth_cookie=" + uid + "; path=/";

        var aCookie = cookieManager.get('auth_cookie');
        if (!aCookie || typeof aCookie != 'object')	aCookie = {};
		aCookie[auth_array.username] = uid;
		cookieManager.set ('auth_cookie',aCookie,-1);

		request.COOKIE = [{VALUE:uid}];
	}
	*/

	/* send data to server & set response handler */
	if(!sDataSet){
		var aData = this.create_iq(request,'','','set');
		if (this.error) return false;
		aData = aData["IQ"][0]["ATTRIBUTES"]["SID"];
		dataSet.add('main',['sid'],aData);

		return aData;
	}
	else
		this.create_iq(request,[this,'response',['login',sDataSet,sDataPath]],'','set');

	return true;
};


_me.get_digest = function(sHashId,sMethod,aHandler){
    var request = {METHOD:[{VALUE:sMethod}]};
	if (sMethod == 'SIP'){
		request.HASHID = [{VALUE:sHashId.hash}];
		if (sHashId.command && sHashId.command!==null){
			request.COMMAND = [{VALUE:sHashId.command}];
			request.URI = [{VALUE:sHashId.uri}];
		}
	}
	else
		request.HASHID = [{VALUE:sHashId}];

	if (aHandler)
		this.create_iq(request,[this,'response',['digest','','',aHandler]],'','get');
	else{
		try{
		var aData = this.create_iq(request,'','','get');
			aData = aData["IQ"][0]["QUERY"][0]["DIGEST"][0]["VALUE"];
			return aData;
		}
		catch(r){
			return false;
		}
	}
};


_me.logout = function(bIgnoreResponse){

	if (!dataSet.get("main",['sid'])) return true;

	//Login save state 0
	if (cookieManager.get('LoginState') == '3')
		cookieManager.set('LoginState','2');

	if (bIgnoreResponse) {
		this.create_iq(null,[this,'_void'],'','set');
		return true;
	}
	else
		this.create_iq(null,'','','set');

	dataSet.remove("main",['sid'],true);
	return true;
};

_me._forgot = function(email,csid,captcha){

	var aRequest = {"FORGOT" :[{"VALUE":email}],
					"CAPTCHA":[{'ATTRIBUTES':{'UID':csid},"VALUE":captcha}],
					"SUBJECT":[{"VALUE":GWOthers.getItem('FORGOT_SETTINGS','subject') || getLang('FORGOT_PASS::SUBJECT')}],
					"MESSAGE":[{"VALUE":GWOthers.getItem('FORGOT_SETTINGS','mail') || getLang('FORGOT_PASS::EMAIL')}]};

	var aResponse = (this.create_iq(aRequest,'','','set')).IQ[0];

	if (aResponse.ATTRIBUTES.TYPE == 'result'){
		var emails = [];
		for(var i in aResponse.QUERY[0].EMAIL)
			emails.push(aResponse.QUERY[0].EMAIL[i].VALUE);

		return emails;
	}
	else
	if (aResponse.ATTRIBUTES.TYPE == 'error')
		return aResponse.ERROR[0].ATTRIBUTES.UID;
	else
		return false;
};

_me._void = function(){};

_me.digest = function(user,pass){
	// encrypt password
	var rsa = new RSAKey();
		rsa.setPublic(this.hashid({"username":user}), '10001');
	return rsa.encrypt(pass);
};

/**
 * @brief: retrieve HashID from server
 * @query: <iq><query xmlns="webmail:iq:auth"><username>admin</username><method>rsa</method></query></iq>
 **/
_me.hashid = function(auth_array)
{
	/* prepare data for request */
	var request={"USERNAME":[{"VALUE":auth_array['username']}],"METHOD":[{"VALUE":'RSA'}]};

	/* send data to server & get response */
	var response = this.create_iq(request);

	/* return HashId */
	return response["IQ"][0]["QUERY"][0]["HASHID"][0]["VALUE"];
};

_me.response=function(aData,sMethodName,sDataSet,sDataPath,aHandler)
{
	if (sDataSet == 'main')
		sDataPath = ['sid'];
	else
		sDataPath = sDataPath || ['sid'];

	switch(sMethodName){
	case 'login':
		dataSet.add(sDataSet,sDataPath,aData['Array']["IQ"][0]["ATTRIBUTES"]["SID"]);
		break;
	case 'logout':
		dataSet.add(sDataSet,sDataPath,'');
		break;
	case 'digest':
		executeCallbackFunction(aHandler, aData['Array']["IQ"][0]["QUERY"][0]["DIGEST"][0]["VALUE"]);
	}
};

/////////////////////////
var auth = new wm_auth();