function wm_folders(){
	this.xmlns = 'folders';
};

wm_folders.inherit(wm_generic);
var _me = wm_folders.prototype;

//********************************************************************************
//Základní SET funkce pro pro přidávání a update folderu                    * OK *
//********************************************************************************
//Vstup: aFolderInfo ... asociativní pole klíčů:
//  povinné:   'aid':account_id, 'name':jméno folderu
//  nepovinné: 'fid':folder_id, 'type':typ folderu
//********************************************************************************

_me.add = function(aFolderInfo,sDataSet,aDataPath,aHandler)
{

	//Máme definované account_id a jméno folderu?
	if (!aFolderInfo['aid'])
		return false;

	var sUpdDataSet = sDataSet || 'folders';

	var aRequest;

	//Budeme složku editovat či přidávat?
	if (aFolderInfo['fid']){
		if (!aFolderInfo['default'] && !aFolderInfo['name'] && typeof aFolderInfo['channel'] == 'undefined' && !aFolderInfo['acl'])
	        return false;

		//Vytvoříme XML dotaz pro editaci folderu
		// <ACCOUNT UID='aid'><FOLDER UID='fid' ACTION='edit'><NAME>nové jméno folderu</NAME></FOLDER></ACCOUNT>
		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"edit"}}]}]};
	}
	else{
	    if (!aFolderInfo['name'])
	        return false;

		//V případě zadaného typu folderu ověříme, zdali je podporovaný, jinak ho nastavíme na 'mail'
		if (aFolderInfo['type']){
			if (inArray(['mail','contact','event','journal','note','task','file','m','r','c','e','j','n','t','f'],aFolderInfo['type'].toLowerCase()) < 0)
				return false;
				
			//create Virtual Account RSS if doesn't exists yet
			if (aFolderInfo['type'] == 'R' && !dataSet.get('accounts',[sPrimaryAccount+'_rss']))
				dataSet.add('accounts',[sPrimaryAccount+'_rss'],{DESCRIPTION:'RSS',TYPE:'rss'});
		}
		else
			aFolderInfo['type'] = 'mail';

		//Vytvoříme XML dotaz pro přidání folderu
		// <ACCOUNT UID='aid'><FOLDER ACTION='add'><NAME>jméno folderu</NAME><TYPE>typ folderu</TYPE></FOLDER></ACCOUNT>
		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"ACTION":"add"},"TYPE":[{"VALUE":aFolderInfo['type']}]}]}]};
	}

	if (aFolderInfo['name'] && aFolderInfo['fid']!=aFolderInfo['name'])
		aRequest.ACCOUNT[0].FOLDER[0].NAME = [{'VALUE':aFolderInfo['name']}];

	if (aFolderInfo['default'])
		aRequest.ACCOUNT[0].FOLDER[0].DEFAULT = [{'VALUE':aFolderInfo['default']}];

	//Virtual Folders
	if (aFolderInfo.virtual){

		aRequest.ACCOUNT[0].FOLDER[0].VIRTUAL = [];

		if (aFolderInfo.virtual.folders){
			aRequest.ACCOUNT[0].FOLDER[0].VIRTUAL = [{FOLDERS:[{'FOLDER':[]}]}];

			var tmp;
			for(var i in aFolderInfo.virtual.folders){
				tmp = {'VALUE':i};
				if (aFolderInfo.virtual.folders[i])
					tmp.ATTRIBUTES = {'PRIMARY':"true"};

				aRequest.ACCOUNT[0].FOLDER[0].VIRTUAL[0].FOLDERS[0].FOLDER.push(tmp);
			}
		}
	}

	if (aFolderInfo['acl']){
		//<right email="{group_email}">{rights}</right>

		var aGrp = [];
		for (var i in aFolderInfo['acl'])
			aGrp.push({VALUE:aFolderInfo['acl'][i].join(''),ATTRIBUTES:{EMAIL:i}});

		if (aGrp.length)
			aRequest.ACCOUNT[0].FOLDER[0].ACL = [{RIGHT:aGrp}];
		else
		    aRequest.ACCOUNT[0].FOLDER[0].ACL = [{VALUE:''}];
	}

	if (typeof aFolderInfo['channel'] != 'undefined')
		aRequest.ACCOUNT[0].FOLDER[0].CHANNEL = [{"VALUE":aFolderInfo['channel']}];

	//Pracujeme synchronně či asynchronně?
	if (sDataSet || aHandler){
		//AJAX...
		this.create_iq(aRequest,[this,'response',['add',sDataSet,aDataPath,aFolderInfo,aHandler]],'','set',aFolderInfo['aid']);
		return true;
	}
	else{
		var aResponse = this.create_iq(aRequest,'','','set');

		try{
			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
				return true;
		}
		catch(e){}

		return false;
	}
};

//********************************************************************************
//Základní GET funkce pro listování folderů                                 * OK *
//********************************************************************************
//Vstup: aFoldersInfo ... asociativní pole klíčů:
//  povinné:   'aid':account_id
//********************************************************************************

_me.list = function(aFoldersInfo,sDataSet,aDataPath,aHandler)
{
	//Máme definované account_id?
	if (!aFoldersInfo['aid'])
		return false;

	//Vytvoříme XML dotaz pro listování folderu v accountu
	// <ACCOUNT UID='aid'/>
	var aRequest={"ACCOUNT":[{"ATTRIBUTES":{"UID":aFoldersInfo['aid']}}]};

    if (aFoldersInfo['fid'])
        aRequest.ACCOUNT[0].FOLDER = [{"ATTRIBUTES":{"UID":aFoldersInfo['fid']}}];

	//Pracujeme synchronně či asynchronně?
	if (!sDataSet && !aHandler)
		return this.parse(this.create_iq(aRequest));
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['list',sDataSet,aDataPath,'',aHandler]],'','get',aFoldersInfo['aid']);
		return true;
	}
};

/**
 * funkce vraci Typ folderu
 * (nebo False)
 **/
_me.getType = function (aFolderInfo){
    var aid = aFolderInfo.aid || aFolderInfo[0];
    var fid = aFolderInfo.fid || aFolderInfo[1];
    
	return dataSet.get('folders',[aid,fid,'TYPE']) || 'X';
};

/**
 * funkce vraci Rights pro folder
 * (Prava na folder samotny)
 *
 		l(folder read)
		k(folder write)
		[m(folder modify)]
		x(folder delete)
		a(admin/owner)
 **/
_me.getRights = function (aFolderInfo,isRight){

	if (Cookie){
		var tmp = Cookie.get(['rights',aFolderInfo.aid,aFolderInfo.fid]);
		
		if (tmp){
			var aRights = {};
			for (var i in tmp){
				switch(tmp[i]){
					case 'l':
						aRights.read = true;
						break;
					case 'k':
						aRights.write = true;
						break;
					case 'x':
						aRights.modify = true;
						aRights.remove = true;
						break;
					case 'a':
						aRights.owner = true;
						break;
				}
			}
			
			if (!aRights.remove && aFolderInfo.fid.indexOf('~')==0 && aFolderInfo.fid.indexOf('@')>0 && aFolderInfo.fid.indexOf('/')<0)
				aRights.remove = true;

			if (isRight)
				return aRights[isRight] || false;

			return aRights;
		}
	}

	if (isRight)
		return true;
	else
		return {owner:true,read:true,write:true,modify:true,remove:true};
};
/**
 * funkce vraci Access pro Items ve folderu
        r(items read)
		i(items write)
		w(items modify)
		t(items delete)
 **/
_me.getAccess = function (aFolderInfo,isRight){

	if (Cookie){
		var tmp = Cookie.get(['rights',aFolderInfo.aid,aFolderInfo.fid]);
		if (tmp){
			var aRights = {};
			for (var i in tmp){
				switch(tmp[i]){
					case 'r':
						aRights.read = true;
						break;
					case 'i':
						aRights.write = true;
						break;
					case 'w':
						aRights.modify = true;
						break;
					case 't':
						aRights.remove = true;
						break;
				}
			}

			if (isRight)
				return aRights[isRight] || false;

			return aRights;
		}
	}

	return {owner:true,read:true,write:true,modify:true,remove:true};

/*
	var aRights = {read:false,write:false,modify:false,remove:false};
	if(!aFolderInfo.aid || !aFolderInfo.fid) return isRight?false:aRights;

	var sRights = dataSet.get('folders',[aFolderInfo.aid,aFolderInfo.fid,'ACCESS']);

	if (sRights){
		if (sRights.indexOf('r')>-1) aRights['read'] = true;
		if (sRights.indexOf('w')>-1) aRights['write'] = true;
		if (sRights.indexOf('m')>-1) aRights['modify'] = true;
		if (sRights.indexOf('d')>-1) aRights['remove'] = true;
	}
	else
		return this.getRights(aFolderInfo,isRight);

	if (isRight)
		return aRights[isRight] || false;

	return aRights;
*/
};

/**
 * funkce vraci opravdove jmeno systemoveho folderu v pripade ze nebyl
 * nalezen ale existuje jiny lisici se pouze v CaSe
 *  @param: sDataSet - optional
 **
_me._getDefaultFolders = function(sDataSet){

    var aFolders = dataSet.get(sDataSet || 'folders');
	var aFolder = ['trash','drafts','sent'];
	
    if (!aFolders) return aFolder;
	
	var sItem = '',test = {},out = {};
	storage.library('gw_others');

	for(var i in aFolder){
		sItem = GWOthers.getItem('DEFAULT_FOLDERS', aFolder[i]);
		sItem = sItem.substr(sItem.indexOf('/')+1);

		if (!aFolders[sPrimaryAccount][sItem])
			test[aFolder[i]] = [new RegExp('^'+sItem.quoteMeta()+'$', "i"),sPrimaryAccount+'/'+sItem];
		else
		    out[aFolder[i]] = sPrimaryAccount+'/'+sItem;
	}

	if (!count(test))
		return out;

	//search for folders
	for (var itm in aFolders){
		for(var itm2 in test){
			if (itm.match(test[itm2][0])){
				out[itm2] = sPrimaryAccount+'/'+itm;
				test[itm2] = null;
				delete test[itm2];
			}
		}
	}
	
	//append not existing folders
	for(var itm2 in test)
        out[itm2] = test[itm2][1];

	//save to settings
	storage.library('gw_others');
	GWOthers.set('DEFAULT_FOLDERS',out,'storage');

	return out;
};
*/
//********************************************************************************
//Základní SET funkce pro pro mazání folderu                                * OK *
//********************************************************************************
//Vstup: aFolderInfo ... asociativní pole klíčů:
//  povinné:   'aid':account_id, 'fid':folder_id
//********************************************************************************
_me.remove = function(aFolderInfo,sDataSet,aDataPath)
{
	//Máme definovaná account_id a folder_id?
	if (!aFolderInfo['aid'] || !aFolderInfo['fid'])
		return false;

	//Vytvoříme XML dotaz pro výmaz folderu
	// <ACCOUNT UID='aid'><FOLDER UID='fid' ACTION='delete'/></ACCOUNT>
	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"delete"}}]}]};
	

	//Pracujeme synchronně či asynchronně?
	if (!sDataSet){
		var aResponse = this.create_iq(aRequest,'','','set');
		try{
			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
            	return true;
		}
		catch(e){}

		return false;
	}
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['remove',sDataSet,aDataPath,aFolderInfo]],'','set',aFolderInfo['aid']);
		return true;
	}
};

//********************************************************************************
//Základní SET funkce pro pro vyprázdnění folderu                           * OK *
//********************************************************************************
//Vstup: aFolderInfo ... asociativní pole klíčů:
//  povinné:   'aid':account_id, 'fid':folder_id
//********************************************************************************

_me.empty = function(aFolderInfo,sDataSet,aDataPath,aDestination)
{
	//Máme definovaná account_id a folder_id?
	if (!aFolderInfo['aid'] || !aFolderInfo['fid'])
		return false;

	//Vytvoříme XML dotaz pro vyprázdnění folderu
	// <ACCOUNT UID='aid'><FOLDER UID='fid' ACTION='empty'/></ACCOUNT>
	var aRequest;
	if (Is.Defined(aDestination))
		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"empty"},"ACCOUNT":[{"VALUE":aDestination['aid']}],"FOLDER":[{"VALUE":aDestination['fid']}]}]}]};
	else
		aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"empty"}}]}]};

	//Pracujeme synchronně či asynchronně?
	if (!sDataSet){
		var aResponse = this.create_iq(aRequest,'','','set');

		try{
			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result'){
				dataSet.add('items',[aFolderInfo['aid'],aFolderInfo['fid']],{});
				return true;
			}
		}
		catch(e){}

		return false;
	}
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['empty',sDataSet,aDataPath,aFolderInfo]],'','set',aFolderInfo['aid']+'/'+aFolderInfo['fid']);
		return true;
	}
};

/**
 *
 * @IQ Example: <folder uid="{uid}" action="edit"><subscribed>{0|1}</subscribed></folder>
 **/
_me.subscribe = function(aFolderInfo,sDataSet,aDataPath,bSync){
	var sDataSet = sDataSet || 'folders';
	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":"edit"},"SUBSCRIBED":[{"VALUE":bSync?1:0}]}]}]};

	//AJAX...
	this.create_iq(aRequest,[this,'response',[bSync?'sync':'unsync',sDataSet,aDataPath,aFolderInfo,bSync]],'','set',aFolderInfo['aid']+'/'+aFolderInfo['fid']);
	return true;
};


_me.markItemsRead = function(aFolderInfo,sDataSet,aDataPath,bRead)
{
	//Máme definovaná account_id a folder_id?
	if (!aFolderInfo['aid'] || !aFolderInfo['fid'])
		return false;

	//Vytvoříme XML dotaz pro výmaz folderu
	// <ACCOUNT UID='aid'><FOLDER UID='fid' ACTION='delete'/></ACCOUNT>
	var sType = bRead ? 'markasread' : 'markasunread';
	var aRequest = {"ACCOUNT":[{"ATTRIBUTES":{"UID":aFolderInfo['aid']},"FOLDER":[{"ATTRIBUTES":{"UID":aFolderInfo['fid'],"ACTION":sType}}]}]};
	aFolderInfo['bRead'] = bRead;
	
	//Pracujeme synchronně či asynchronně?
	if (!sDataSet){
		var aResponse = this.create_iq(aRequest,'','','set');

		try{
			if (aResponse['IQ'][0]['ATTRIBUTES']['TYPE'] == 'result')
			return true;
		}
		catch(e){}

		return false;
	}
	else{
		//AJAX...
		this.create_iq(aRequest,[this,'response',['markread',sDataSet,aDataPath,aFolderInfo]],'','set',aFolderInfo['aid']);
		return true;
	}
};

//********************************************************************************
//Pomocná funkce realizující asynchronní obsluhu odpovědi
//********************************************************************************

_me.response = function(aResponse,sMethodName,sDataSet,aDataPath,aFolderInfo,aHandler)
{
	var aXMLResponse = aResponse['Array'];
	var aIQAttribute = aXMLResponse['IQ'][0]['ATTRIBUTES'];

	//pri IMAP erroru
	try{
		if (aIQAttribute['TYPE'] == 'error'){

			var str, att;
			try{
				att = aXMLResponse.IQ[0].ERROR[0].ATTRIBUTES.UID;
				str = aXMLResponse.IQ[0].ERROR[0].VALUE;
			}
			catch(e){
				str = att = '';
			}

			if (att.toLowerCase() == 'imap_internal')
				alert ((aIQAttribute['UID']?aIQAttribute['UID']+"\n":'')+str.unescapeHTML());
		}
	}
	catch(e){}

	switch(sMethodName){
    case 'unsync':
	case 'sync':
		if (aIQAttribute['TYPE'] == 'result')
			dataSet.add('folders', [aFolderInfo['aid'],aFolderInfo['fid'],'SYNC'],sMethodName == 'sync'?1:0, true);
		break;
		
	case 'add':
		//pri erroru stahnout znova strom
		if (aIQAttribute['TYPE'] == 'error'){
       		try{
				this.list({'aid':aIQAttribute['UID']},sDataSet,aDataPath);
			}
			catch(e){}
			return true;
		}
        //acl cookie update
		else
		if (Cookie && typeof aFolderInfo.name != 'undefined' && typeof aFolderInfo.aid != 'undefined'){

			/*
			<iq sid="77b529e0d5686f31e84adc6b4d571a2a" uid="admin@merakdemo .com" type="result">
				<query xmlns="webmail:iq:folders">
					<account uid="admin@merakdemo.com">
						<folder uid ="Events" rights="riwtlkxa"/>
					</account>
				</query>
			</iq>
			*/
		
			try{
                var tmp = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0];
                if (tmp.ATTRIBUTES.UID == aFolderInfo.aid && tmp.FOLDER[0].ATTRIBUTES.UID == aFolderInfo.name)
					Cookie.set(['rights',aFolderInfo.aid,aFolderInfo.name], tmp.FOLDER[0].ATTRIBUTES.RIGHTS?tmp.FOLDER[0].ATTRIBUTES.RIGHTS.split(''):'');
			}
			catch(r){}
		}

/*
[aid](string) = admin@merakdemo.com
[fid](string) = SentXX
[name](string) = aaaa/SentXX
*/

		//edit
		if	(aFolderInfo['fid'] && ((aFolderInfo['name'] && aFolderInfo['fid'] != aFolderInfo['name']) || typeof aFolderInfo['channel'] != 'undefined') || aFolderInfo['default']){
			var tmp,
				sName = aFolderInfo['name'] || aFolderInfo['fid'],
				aDataSet = dataSet.get(sDataSet,[aFolderInfo['aid']]);

			for(var i in aDataSet){
				//folder
				if (i == aFolderInfo['fid']){
					tmp = aDataSet[i];

					if (typeof aFolderInfo['channel']!='undefined')
						tmp.CHANNEL = aFolderInfo['channel'];
						
					delete aDataSet[i];
					aDataSet[sName] = tmp;
				}
				else
				//subfolder
				if (i.indexOf(aFolderInfo['fid']+'/') === 0){
					tmp = aDataSet[i];

					delete aDataSet[i];
					aDataSet[sName + i.substr(aFolderInfo['fid'].length)] = tmp;
				}
			}
			
			if (aFolderInfo['aid'] == sPrimaryAccount && aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].DEFAULT){
				var tmp_type = {C:'CONTACTS',E:'EVENTS',T:'TASKS',N:'NOTES',J:'JOURNAL',F:'FILES',D:'DRAFTS',H:'TRASH',S:'SENT'}[aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].DEFAULT[0].VALUE];

				//remove old default
				var def = GWOthers.getItem('DEFAULT_FOLDERS',tmp_type);
				if (def){
					def = def.split("/");
				    def = {aid:def.shift(),fid:def.join('/')};
					if (dataSet.get(sDataSet,[def.aid,def.fid]))
						dataSet.remove(sDataSet,[def.aid,def.fid,'DEFAULT'],true);
				}

				//set new default
				GWOthers.setItem('DEFAULT_FOLDERS',tmp_type,aFolderInfo['aid']+'/'+sName);
				aDataSet[sName].DEFAULT = aXMLResponse.IQ[0].QUERY[0].ACCOUNT[0].FOLDER[0].DEFAULT[0].VALUE;
			}

			var srt = {};
				srt[aFolderInfo['aid']] = aDataSet;

			//	bNoRefresh = true protoze pracujeme primo s datasetem, proto po add neni zmena dat (add je jen pro sychr)
			//	Notne vyvolat refresh manualne
			dataSet.add(sDataSet,[aFolderInfo['aid']],this.sort(srt)[aFolderInfo['aid']],true);
			dataSet.update(sDataSet,[aFolderInfo['aid']]);
		}
		//add
		else
		if (!aFolderInfo['fid'] && aFolderInfo['name']){                                                                                                     //,NAME:aFolderInfo['name'],TOTAL:0
			var out = {TYPE:aFolderInfo['type']};

			if (typeof aFolderInfo['channel'] != 'undefined')
				out.CHANNEL = aFolderInfo['channel'];

			var tmp = dataSet.get(sDataSet,[aFolderInfo['aid']],true) || {};
			    tmp[aFolderInfo['name']] = out;

			var srt = {};
				srt[aFolderInfo['aid']] = tmp;

			dataSet.add(sDataSet,[aFolderInfo['aid']],this.sort(srt)[aFolderInfo['aid']]);
		}

		if (typeof aHandler == 'object')
			executeCallbackFunction(aHandler,aFolderInfo);

		return false;

	case 'list':

		try{
			if (aIQAttribute['UID']){
				var aData = this.parse(aXMLResponse,true);

				if (aData[sPrimaryAccount])
					WMAccounts.__mapfolders(aData[sPrimaryAccount]);

				if (sDataSet)
					dataSet.add(sDataSet, [aIQAttribute['UID']], aData[aIQAttribute['UID']]);

				if (typeof aHandler == 'object')
					executeCallbackFunction(aHandler, aData);

				return true;
			}
		}
		catch(e){}

		return false;

	case 'remove':
		try{
			if (aIQAttribute['TYPE'] != 'result' && aIQAttribute['UID']){
				this.list({'aid':aIQAttribute['UID']},sDataSet,aDataPath);
				return true;
			}
		}
		catch(e){}

		//ASYNC DataSet change
		var blank = true;
		var bPerfm = false;
		var aDataSet = dataSet.get(sDataSet,[aFolderInfo['aid']]);

		for(var i in aDataSet){
			if (i == aFolderInfo['fid'] || i.indexOf(aFolderInfo['fid']+'/') === 0){
				bPerfm = true;
				delete aDataSet[i];
			}
			else
				blank = false;
		}

		//	bNoRefresh = true protoze pracujeme primo s datasetem, proto po add neni zmena dat (add je jen pro sychr)
		//	Notne vyvolat refresh manualne
		if (blank && aFolderInfo['aid'] == sPrimaryAccount+'_rss'){
			dataSet.remove('accounts',[aFolderInfo['aid']],true);
			dataSet.remove(sDataSet,[aFolderInfo['aid']]);
		}
		else
		if(bPerfm){
			dataSet.add(sDataSet,[aFolderInfo['aid']],aDataSet,true);
			dataSet.update(sDataSet,[aFolderInfo['aid']]);
		}

		return false;

	case 'empty':
		try{
			if (aIQAttribute['TYPE'] != 'result' && aIQAttribute['UID']) {
				var aFolder = Path.split(aIQAttribute['UID']);
				this.list({'aid':aFolder[0]},sDataSet,aDataPath);

				// je potreba znovu vylistovat otevreny folder?
				var aItems = dataSet.get('items');
				for(var sAccId in aItems)
					for(var sFolId in aItems[sAccId]);
						if (sAccId == aFolder[0] && sFolId == aFolder[1])
							WMItems.list({'aid':sAccId,'fid':sFolId,'values':items.default_values('M')},'items');

				return true;
			}
			else
			if (aIQAttribute['UID']){
				var aFolder = Path.split(aIQAttribute['UID']);

				// je potreba promazat Items?
				var aItems = dataSet.get('items');
				for(var sAccId in aItems)
					for(var sFolId in aItems[sAccId]);
						if (sAccId == aFolder[0] && sFolId == aFolder[1])
							dataSet.add('items',[aFolder[0],aFolder[1]],{});
			}
		}
		catch(e){}

		//ASYNC DataSet change
		if (dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'])>0)
			dataSet.add(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'],'0'); // ,true

		return false;


	case 'markread':
		if (aIQAttribute['TYPE'] == 'error')
			return true;

		// refresh folder list
		if (aFolderInfo.bRead){
			if (dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'])>0)
				dataSet.add(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'],'0');
		}
		else {
			var nCount = 0;
			if (Is.Defined(aXMLResponse['IQ'][0]['QUERY'][0]['RECENT'])) {
				nCount = parseInt(aXMLResponse['IQ'][0]['QUERY'][0]['RECENT']);
				if (!Is.Number(nCount))
					nCount = 0;
			}

			var i = dataSet.get(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT']);
				i = i>0?i:0;

			if (i != nCount)
				dataSet.add(sDataSet,[aFolderInfo['aid'],aFolderInfo['fid'],'RECENT'],nCount.toString());
		}

		// je potreba aktualizovat Items?
		var aItems = dataSet.get('items');
		for(var sAccId in aItems)
			for(var sFolId in aItems[sAccId]);
				if (sAccId == aFolderInfo['aid'] && sFolId == aFolderInfo['fid'])
					gui.frm_main.main.list._serverSort();

		return true;
	}
};

/**
 * Pomocná funkce realizující abecední seřazení folderů
 *
 * @param:  aFolders - array of folders
 *          bRights  - apply rights (TRUE only if aFolders are fresh data from server!!)
 **/
_me.sort = function(aFolders)
{
	function sort(a,b){
		var sA;
		var sB;

		if (Is.String(a['TITLE']))
			sA = a['TITLE'].toLowerCase();
		else
			sA = a['TITLE'];

		if (Is.String(b['TITLE']))
			sB = b['TITLE'].toLowerCase();
		else
			sB = b['TITLE'];

		if (sA>sB) return 1;
		if (sA<sB) return -1;
		return 0
	};

	//Rozdělíme foldery mezi Inbox, IMAP a GW
	var aResult = {};
	var aResultFolFrame,aFolder,aFolderSplit;

	var aSortInbox, aSortOthers, aSortShared, aSortPublic/*, tmpPublic*/;

	var sTitle;
	var bSpamFolders = false;
	var sRights;

	//get common folders from langs
	var aCommon = getLang('COMMON_FOLDERS');

	for(var sAccId in aFolders){
		//Seřazovací pole nulujeme
		aSortInbox = [];
		aSortOthers = [];
		aSortShared = [];
		aSortPublic = [];

		//Roztřídění folderů podle typu
		for(var sFolId in aFolders[sAccId]){

			aFolderSplit = sFolId.split('/');

			//INBOX
			if (aFolderSplit[0] == 'INBOX'){
				if (aFolderSplit[0] == sFolId)
					aFolders[sAccId][sFolId]['NAME'] = aCommon['INBOX'];

				aFolderSplit[0] = aCommon['INBOX'];
				aSortInbox.push({'FOLDER':sFolId,'TITLE':aFolderSplit.join('/')});
			}
			else
			//VIRTUAL FOLDERS
			if (aFolderSplit[0] == '__@@VIRTUAL@@__'){
				if (aFolderSplit[0] == sFolId)
					aFolders[sAccId][sFolId]['NAME'] = aCommon['VIRTUAL-FOLDERS'];

				aFolderSplit[0] = aCommon['VIRTUAL-FOLDERS'];
				aSortOthers.push({'FOLDER':sFolId,'TITLE':aFolderSplit.join('/')});
			}
			else
			if (aFolderSplit[0] == 'SPAM_QUEUE'){
				aFolders[sAccId][sFolId]['NAME'] = aCommon['SPAM_QUEUE-'+ aFolderSplit[1].toUpperCase()];
				aSortOthers.push({'FOLDER':sFolId,'TITLE':sFolId});
			}
			else
			if (aFolderSplit[0] == 'Quarantine' && aFolders[sAccId][sFolId].TYPE == 'Q'){
				aFolders[sAccId][sFolId]['NAME'] = aCommon['QUARANTINE'];
				aSortOthers.push({'FOLDER':sFolId,'TITLE':sFolId});
			}
/*

			else
			//COMMON FOLDERS
			if ((sTitle = aCommon[aFolderSplit[0].toUpperCase()]))
			{
				if (aFolderSplit[0] == sFolId)
					aFolders[sAccId][sFolId]['NAME'] = sTitle;
				else
				if (aFolderSplit[0] == 'SPAM_QUEUE')
					aFolders[sAccId][sFolId]['NAME'] = aCommon[(aFolderSplit[0]+'-'+aFolderSplit[1]).toUpperCase()];

				aFolderSplit[0] = sTitle;
				
				aSortOthers.push({'FOLDER':sFolId,'TITLE':aFolderSplit.join('/')});
			}
*/
			//SHARED & PUBLIC & OTHER FOLDERS
			else{
				if (sFolId.indexOf('~')==0 && sFolId.indexOf('@')>0 && sFolId.indexOf('/')<0)
					aFolders[sAccId][sFolId].NAME = sFolId.substr(1);

				if ((aFolders[sAccId][aFolderSplit[0]] && aFolders[sAccId][aFolderSplit[0]].TYPE == 'A') || (aFolderSplit[0].indexOf('~')==0 && aFolderSplit[0].indexOf('@')>0))
					aSortShared.push({'FOLDER':sFolId,'TITLE':sFolId});
				else
					aSortOthers.push({'FOLDER':sFolId,'TITLE':sFolId});
			}
		}
		
		//Rozdělené foldery setřídíme
		aSortInbox.sort(sort);
		aSortOthers.sort(sort);
        aSortShared.sort(sort);
        
		//Add missing folders (in case of shared subfolders)
		var fld,tmp;
		for (var i = 0; i<aSortShared.length; i++){
			if (aSortShared[i].FOLDER.indexOf('/')>-1){
				fld = aSortShared[i].FOLDER.split('/'),
				tmp = '', k = 0;

				for(var j=0;j<=fld.length;j++){
					fld.pop();
					tmp = fld.join('/');

					if (aSortShared[i-1].FOLDER.indexOf(tmp)==0)
					    break;

						aSortShared.splice(i,0,{FOLDER:tmp});
                    k++;
				}

				i+=k;
			}
		}

		//Vytvoření seřazených folderů v rámci accountu
		aResultFolFrame = {};

		for(var n in aSortInbox){
			aFolder = aFolders[sAccId][aSortInbox[n]['FOLDER']];
			aResultFolFrame[aSortInbox[n]['FOLDER']] = {};
			for(var sItem in aFolder)
				aResultFolFrame[aSortInbox[n]['FOLDER']][sItem] = aFolder[sItem];
		}

		for(var n in aSortOthers){
			aFolder = aFolders[sAccId][aSortOthers[n]['FOLDER']];
			aResultFolFrame[aSortOthers[n]['FOLDER']] = {};
			for(var sItem in aFolder)
				aResultFolFrame[aSortOthers[n]['FOLDER']][sItem] = aFolder[sItem];
		}


		for(var n in aSortShared){
			aFolder = aFolders[sAccId][aSortShared[n]['FOLDER']];

			if (!aFolder && aSortShared[n].FOLDER.indexOf('~')==0 && aSortShared[n].FOLDER.indexOf('@')>0 && aSortShared[n].FOLDER.indexOf('/')<1)
				aResultFolFrame[aSortShared[n]['FOLDER']] = {TYPE:'VA'};
			else
				aResultFolFrame[aSortShared[n]['FOLDER']] = {};


			for(var sItem in aFolder)
				aResultFolFrame[aSortShared[n]['FOLDER']][sItem] = aFolder[sItem];
		}

		aResult[sAccId] = aResultFolFrame;
	}

	return aResult;
};

//********************************************************************************
//Pomocná funkce realizující převod z "XML" pole do Dataset struktur        * OK *
//********************************************************************************

_me.parse = function(aData)
{
	try
	{
		var aAccFrame = aData['IQ'][0]['QUERY'][0]['ACCOUNT'][0],
			sAccId = aAccFrame['ATTRIBUTES']['UID'],
			aFolFrame = aAccFrame['FOLDER'],
			sFolId,
			aResult = {},
			aResultAccFrame = {},
			aResultFolFrame,
			aRights,
        	acl = false;
        	
		//Map GW folders
		var aTmp = {C:'CONTACTS',E:'EVENTS',T:'TASKS',N:'NOTES',J:'JOURNAL',F:'FILES',
					D:'DRAFTS',H:'TRASH',S:'SENT'},
			sFolderType = '';

		for(var nFolNum in aFolFrame){
			aResultFolFrame = {};
            sFolId = aFolFrame[nFolNum]['ATTRIBUTES']['UID'];
            
			for(var sItem in aFolFrame[nFolNum])
                if (sItem == 'ACL'){
					if (aFolFrame[nFolNum][sItem][0] && aFolFrame[nFolNum][sItem][0].RIGHT){
						aResultFolFrame[sItem] = {};
                        aRights = aFolFrame[nFolNum][sItem][0].RIGHT;
						for(var i in aRights)
							aResultFolFrame[sItem][aRights[i].ATTRIBUTES.EMAIL] = aRights[i].VALUE?aRights[i].VALUE.split(''):[];
					}

					acl = true;
                }
				else
				/*
				//uz je prepsano do attributu
				if (sItem == 'RIGHTS'){
					//Update RIGHTS in cookie     - is updated in Items and ACL response only!
					if (typeof Cookie == 'object'){
					    var sRights = aFolFrame[nFolNum][sItem][0]['VALUE'];
						Cookie.set(['rights',sAccId,sFolId], sRights?sRights.split(''):'');
					}
				}
				else
				*/
				//&& !aFld[aFolFrame[nFolNum].TYPE[0].VALUE] && aFolFrame[nFolNum].ATTRIBUTES.UID.indexOf('/')<0 &&
				if (sItem == 'DEFAULT' && aFolFrame[nFolNum].ATTRIBUTES.UID.indexOf('~')<0){
					sFolderType = aFolFrame[nFolNum].TYPE[0].VALUE;
					if (sFolderType == 'M')
                        sFolderType = aFolFrame[nFolNum][sItem][0].VALUE;

					GWOthers.setItem('DEFAULT_FOLDERS',aTmp[sFolderType],sPrimaryAccount+'/'+aFolFrame[nFolNum].ATTRIBUTES.UID);

					aResultFolFrame[sItem] = aFolFrame[nFolNum][sItem][0]['VALUE'];
				}
                else
                if (sItem == 'VIRTUAL'){
					if (aFolFrame[nFolNum][sItem][0].FOLDERS && aFolFrame[nFolNum][sItem][0].FOLDERS[0].FOLDER){
	                    aResultFolFrame[sItem] = {FOLDERS:[]};
						var aFolers = aFolFrame[nFolNum][sItem][0].FOLDERS[0].FOLDER;
						for(var i in aFolers)
							aResultFolFrame[sItem].FOLDERS[aFolers[i].VALUE] = aFolers[i].ATTRIBUTES && aFolers[i].ATTRIBUTES.PRIMARY?true:false;
					}
				}
				else
				if (sItem != 'ATTRIBUTES')
					aResultFolFrame[sItem] = aFolFrame[nFolNum][sItem][0]['VALUE'];

			aResultAccFrame[sFolId] = aResultFolFrame;
		}
		aResult[sAccId] = aResultAccFrame;

		return aFolFrame && aFolFrame.length>1?this.sort(aResult):aResult;
	}
	catch(e){

		return false;
	}
};

// TODO move this function somewhere else
_me.__emptyFolder = function(sAccId,sFolId,bMoveToTrash)
{
	var aTrashFolder;
	/*
	var nCount = parseInt(dataSet.get('folders',[sAccId,sFolId,'TOTAL']));
	if (!Is.Number(nCount) || nCount <= 0)
		return;
	*/
	if (bMoveToTrash) {
		var aFolData = clone(dataSet.get('folders', [sAccId,sFolId]), true);
		if (aFolData.TYPE == 'M'){
			storage.library('gw_others');
			aTrashFolder = Path.split(GWOthers.getItem('DEFAULT_FOLDERS', 'trash'));
			var sDestAccount = aTrashFolder[0];
			var sDestFolder = aTrashFolder[1];
			var aDestData = clone(dataSet.get('folders', [sDestAccount,sDestFolder]), true);

			if (Is.Defined(aDestData) && Is.Defined(aFolData)) {
				var iRec = parseInt(aFolData['RECENT'] || 0) + parseInt(aDestData['RECENT'] || 0);
				var iOld = parseInt(dataSet.get('folders', [sDestAccount,sDestFolder,'RECENT']) || 0);
				if (iRec != iOld)
					dataSet.add('folders', [sDestAccount,sDestFolder,'RECENT'], iRec.toString());
			}
			aTrashFolder = {'aid':sDestAccount,'fid':sDestFolder};
		}
	}

	if (dataSet.get('folders',[sAccId,sFolId,'RECENT'])>0)
		dataSet.add('folders',[sAccId,sFolId,'RECENT'],'0');

	//Remove mailview if in the same folder
	var aMView = dataSet.get('mailview');
	if (typeof aMView == 'object'){
		for(var mwa in aMView)
			for(var mwf in aMView[mwa]);

		if (mwa == sAccId && mwf == sFolId)
			dataSet.remove('mailview');
	}

	this.empty({'aid':sAccId,'fid':sFolId},'folders','',aTrashFolder);
};


///////////////////////////////
folders = new wm_folders();		// For bacward compatibility
var WMFolders = folders;		// TODO: create real static functions in wm_folders instead of simulating it.