#!/bin/bash

# test if have functions included
if [ -z "$FUNCTIONS_INCLUDED" ]
    then
	echo "Script daemonlib.sh requires to have included functions.sh"
	exit 1
fi

# initial variables

# wait time in seconds before sending SIGKILL to daemon
MAX_WAIT_SECONDS="5"
# wait time in micro seconds == 1/5 second
WAIT_TIME="200000"
# max wait time in microseconds == 5 seconds
MAX_WAIT_TIME=$WAIT_TIME
let "MAX_WAIT_TIME*=${MAX_WAIT_SECONDS}*5"

DAEMON_DESC=""
DAEMON=""

# Function returns daemon description string according to name
# Parameters:
#   $1 - name of dameon
# Returns:
#   Sets variables DAEMON_DESC and DAEMON
#   always 0
#
get_daemon_desc()
{
    case "$1" in
	"icewarpd")
	    DAEMON_DESC="IceWarp Server helper daemon"
	    DAEMON="merakd"
	    ;;
	"control")
	    DAEMON_DESC="IceWarp Server control daemon"
	    DAEMON="control"
	    ;;
	"smtp")
	    DAEMON_DESC="IceWarp Server SMTP daemon"
	    DAEMON="smtp"
	    ;;
	"pop3"|"imap")
	    DAEMON_DESC="IceWarp Server POP3/IMAP4 daemon"
	    DAEMON="pop3"
	    ;;
	"im")
	    DAEMON_DESC="IceWarp Server Messaging daemon"
	    DAEMON="im"
	    ;;
	"cal"|"calendar")
	    DAEMON_DESC="IceWarp Server GroupWare daemon"
	    DAEMON="cal"
	    ;;
	"php"|"phpd")
	    DAEMON_DESC="IceWarp Server PHP"
	    DAEMON="php"
	    ;;
	*)
	    DAEMON_DESC="Unknown daemon '$1'"
	    DAEMON="unknown"
	    return 0
	    ;;
    esac
    return 0
}

startdaemon()
{
    get_daemon_desc "$1"

    pushd "$INSTALLDIR" &>/dev/null

    echo -n "Starting '${DAEMON_DESC}': "
    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export INSTALLDIR
    export PHP_FCGI_MAX_REQUESTS=1000
    export PHP_FCGI_CHILDREN=8

    "${INSTALLDIR}/${DAEMON}" --user=${INSTALLUID} --group=${INSTALLGID} --capabilities=${INSTALLCAP} "--installdir=${INSTALLDIR}" --noservices &> /dev/null
    if [ $? -eq 0 ]
	then
	    echo "OK"
	else
	    echo "FAIL"
    fi

    popd &>/dev/null
    return 0
}

stopdaemon()
{
    get_daemon_desc "$1"

    PID=$(cat "${INSTALLDIR}/var/${DAEMON}.pid" 2> /dev/null)

    echo -n "Shutting down '${DAEMON_DESC}', PID ${PID}: "
    if [ "x" == "x$PID" ]; then
	echo "Can not read PID."
	return 0;
    fi

    WAITED="0"
    while [ true ]
    do
	kill $PID 2> /dev/null
	usleep $WAIT_TIME

	kill -0 $PID 2> /dev/null
	if [ $? -ne 0 ]; then
	    break
	fi

	let "WAITED+=$WAIT_TIME"
	if [ "$WAITED" == "$MAX_WAIT_TIME" ]; then
		echo -n "${MAX_WAIT_SECONDS} seconds timeouted, killing: "
		kill -9 $PID
		break
	fi
    done

    rm -f "${INSTALLDIR}/var/${DAEMON}.pid" 2> /dev/null
    echo "OK"

    # stop PHP when stopping control service
    if [ "$DAEMON" == "control" ]
	then
	    stopdaemon "php"
    fi
    return 0
}

startphp()
{
    # get first argument which should be max PHP processes
    PHP_FCGI_CHILDREN=$1
    # test if PHP_FCGI_CHILDREN is empty string
    if [ "x$PHP_FCGI_CHILDREN" == "x" ]; then
	PHP_FCGI_CHILDREN=8
    fi
    # test if PHP_FCGI_CHILDREN is number
    echo $PHP_FCGI_CHILDREN | grep "[^0-9]" > /dev/null 2>&1
    if [ $? -eq 0 ]; then 
	PHP_FCGI_CHILDREN=8
    fi
    if [ $PHP_FCGI_CHILDREN -eq 0 ]; then
	PHP_FCGI_CHILDREN=8
    fi

    pushd "$INSTALLDIR" &> /dev/null

    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export INSTALLDIR
    export PHP_FCGI_MAX_REQUESTS=1000
    export PHP_FCGI_CHILDREN
    export ASPELL_PREFIX="${INSTALLDIR}/php"

    # need to save PID of php process
    "${INSTALLDIR}/php/php" -c "${INSTALLDIR}/php/php.ini" -b "${INSTALLDIR}/var/phpsocket" 2> "${INSTALLDIR}/logs/phpstartup.log" & echo $! > "${INSTALLDIR}/var/php.pid"

    popd &> /dev/null
    return 0
}

executephp()
{
    # get first argument which is PHP script to execute
    ARGUMENT=$1
    if [ "x$ARGUMENT" == "x" ]; then
	return -1
    fi

    pushd "$INSTALLDIR" &> /dev/null

    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export INSTALLDIR
    export ASPELL_PREFIX="${INSTALLDIR}/php"

    "${INSTALLDIR}/php/php" -c "{INSTALLDIR}/php/php.ini" "$1"
    RET_VAL=$?

    popd &> /dev/null
    return $RET_VAL
}

startpurpleserv()
{
    pushd "$INSTALLDIR" &> /dev/null

    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export LD_LIBRARY_PATH="${INSTALLDIR}/purple/lib:${INSTALLDIR}/purple/plugins:${LD_LIBRARY_PATH}"
    export INSTALLDIR

    # need to start in purpleserv directory
    # need to save PID of purpleserv process
    cd "${INSTALLDIR}/purple" &> /dev/null
    if [ $? -ne 0 ]; then
    	echo "Cannot change to purple directory" &> "${INSTALLDIR}/logs/purple/purpleerror.log"
    	return 1
    fi
    "${INSTALLDIR}/purple/purpleserv" &> "${INSTALLDIR}/logs/purple/purplestartup.log" & echo $! > "${INSTALLDIR}/var/purpleserv.pid"

    popd &> /dev/null

    return 0
}
