#!/bin/bash
#
# IceWarp Server
# (c) IceWarp Ltd. 2008
#
#       All rights reserved.
#
# http://www.icewarp.com
#
# file: functions.sh - include file with basic functions
# version: 1.0
#

# This file is not to be run directly
#
if [ `basename "$0"` == "functions.sh" ]
    then
	echo "This file must be included from script."
	exit 1
fi

if [ -n "$FUNCTIONS_INCLUDED" ]
    then
	echo "This file was already included."
	exit 1
fi

colorize()
{
    if [ "$TERM" = "linux" ] || [ "$TERM" = "xterm" ] || [ "$TERM" = "xterm-color" ] || [ "$TERM" = "screen" ] && [ "$USE_COLORS" = "1" ]
	then
	    GOOD=$'\e[32;01m'
	    WARN=$'\e[33;01m'
	    BAD=$'\e[31;01m'
	    NORMAL=$'\e[0m'
	    HILITE=$'\e[37;01m'
	    BRACKET=$'\e[34;01m'
	else
	    GOOD=$'\e[0;32m'
	    BAD=$'\e[0;31m'
	    NORMAL=$'\e[0m'
	    WARN=$NORMAL
	    HILITE=$NORMAL
	    BRACKET=$NORMAL
    fi
}

doinit()
{
    source $1
    if [ "$InstallDir" != "" ]
	then
	    INSTALLDIR=$InstallDir
    fi
    if [ "$User" != "" ]
	then
	    INSTALLUSER=$User
    fi
    if [ "$Group" != "" ]
	then
	    INSTALLGROUP=$Group
    fi
    if [ "$Version" != "" ]
	then
	    SERVERVERSION=$Version
    fi
    if [ "$Capabilities" != "" ]
	then
	    SERVERCAP=$Capabilities
    fi
    REALINSTALLDIR=$INSTALLDIR
    WIZARDCMDLOG="${INSTALLDIR}/logs/wizard/icewarpserver-wizard-cmd.log"
    SCRIPTDIR="${INSTALLDIR}/scripts"
    TOOL="${INSTALLDIR}/tool"
    SERVER_LD_LIBRARY_PATH="${INSTALLDIR}/lib:${INSTALLDIR}/php/lib:${INSTALLDIR}/avast"
    SERVERCONF="${INSTALLDIR}/config/$2"
    INSTALLUID=$(id -u $INSTALLUSER)
    INSTALLGID=$(id -g $INSTALLUSER)

    FUNCTIONS_INCLUDED="1"
}

preconf()
{
	SERVERCONF=""
    pushd "$(pwd)" > /dev/null
    # search for icewarp.conf
    SEARCHCONF="config/icewarp.conf ../config/icewarp.conf ../../config/icewarp.conf"
    for TEMPCONF in $SEARCHCONF
    do
	if [ -f $TEMPCONF ]; then
		SERVERCONF=$TEMPCONF
		doinit "$SERVERCONF" "icewarp.conf"
		break 1
	fi
    done
    # search for merak.conf
    if [ "$SERVERCONF" == "" ]; then
    	SEARCHCONF="config/merak.conf ../config/merak.conf ../../config/merak.conf"
    	for TEMPCONF in $SEARCHCONF
    	do
			if [ -f $TEMPCONF ]; then
				SERVERCONF=$TEMPCONF
				doinit "$SERVERCONF" "merak.conf"
				break 1
			fi
    	done
    fi
    popd > /dev/null
    colorize
}

regenmerakconf()
{
    echo "## Merak Mail Server configuration file
## this file was generated by regenmerakconf() on ${DATE}
## InstallDir=/path/to/merak
## Merak installation prefix, absolute path, no useful default

InstallDir=${INSTALLDIR}

## Version=number
## Merak Mail Server version, for reference only

Version=${SERVERVERSION}

## User=username
## User to run services as, default=root

User=${INSTALLUSER}

## Group=group
## Group to run services as, default=root

Group=${INSTALLGROUP}

## Capabilities
## capabilities that are set at startup, see
## man cap_from_text
## for more info
## ADVANCED USERS ONLY!

Capabilities=${SERVERCAP}" > $SERVERCONF
}

good()
{
    echo -e "${GOOD}**${NORMAL}\t$*"
}

bad()
{
    echo -e "${BAD}**${NORMAL}\t$*"
}

verybad()
{
    echo "${BAD}************${HILITE}************${BAD}************${NORMAL}"
    echo "${HILITE}$*"
    echo "${BAD}************${HILITE}************${BAD}************${NORMAL}"
    exit 1
}

warn()
{
    echo -e "${WARN}**${NORMAL}\t$*"
}

hilite()
{
    echo -e "${GOOD}**${HILITE}\t$*${NORMAL}"
}

getparam()
{
    echo -e -n "${GOOD}**${NORMAL}\t$1 "
    read PARAM
}

getpassword()
{
    DIFFER="1"
    while [ "$DIFFER" == "1" ]
	do
	    echo -e -n "${GOOD}**${NORMAL}\t$1 "
	    read -s TOKEN
	    good ""
	    echo -e -n "${GOOD}**${NORMAL}\tOnce again to confirm: "
	    read -s TOKEN_CONFIRM
	    good ""

	    if [ "$TOKEN" == "$TOKEN_CONFIRM" ]
		then
		    PASSWORD_PARAM=$TOKEN
		    DIFFER="0"
		else
		    bad ""
		    bad "Passwords don't match."
		    bad "Try again"
		    bad ""
	    fi
	done
}

# Ask question with confirmation
# Parameters:
# $1 - question
# $2 - first choice, it is choosed if user enters empty string
# $3 - second choice
# Returns:
# 0 on error - unknown string entered
# 1 if first choice was selected
# 2 if second choice was selected
ask_with_confirmation()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]
	then
	    return 0
    fi
    LOWER_CHOICE_1=$(echo $2 | tr "A-Z" "a-z")
    LOWER_CHOICE_2=$(echo $3 | tr "A-Z" "a-z")
    echo -e -n "${GOOD}**${NORMAL}\t$1 [${HILITE}$2${NORMAL}/$3]: "
    read PARAM
    PARAM=$(echo $PARAM | tr "A-Z" "a-z")
    if [ "$PARAM" == "" ] || [ "$PARAM" == "$LOWER_CHOICE_1" ]
	then
	    return 1
    fi
    if [ "$PARAM" == "$LOWER_CHOICE_2" ]
	then
	    return 2
    fi
    return 0
}

menu()
{
    if [ -n "$4" ]
	then
	    MENUSCRIPTSDIR=$4
	else
	    MENUSCRIPTSDIR="."
    fi

    IFS_PREV=$IFS
    IFS=$'\n'
    while true
    do
	display_menu "$1" "$2" "$3"
	# quit from menu
	if [ "$PARAM" == "0" ]
	    then
		return
	fi
	# quit from script which uses menu
	if [ "$PARAM" == "Q" ] || [ "$PARAM" == "q" ]
	    then
		exit 0
	fi
	for LINE in `cat $3 | grep "^$2:"`
	    do
		LINECHOICE=$(echo $LINE |cut -d':' -f2)
		NAME=$(echo $LINE | cut -d':' -f3)
		TYPE=$(echo $LINE | cut -d':' -f4)
		if [ "$LINECHOICE" == "$PARAM" ]
		    then
			unset PARAM
			case "$TYPE" in
			    "script"|"SCRIPT")
				good ""
				SCRIPT=$(echo $LINE | cut -d':' -f5)
				pushd $MERAKDIR &>/dev/null
				export SCRIPTDIR
				${SCRIPTDIR}/${MENUSCRIPTSDIR}/${SCRIPT}
				if [ $? -ne 0 ]
				    then
					bad "Error running script!"
				fi
				popd &>/dev/null
				;;
			    "menu"|"MENU")
				MENUROOT=$(echo $LINE | cut -d':' -f5)
				if [ "$MENUROOT" != "QUIT" ]
				    then
					clear
					menu "$NAME" "$MENUROOT" "$3" "$MENUSCRIPTSDIR"
					clear
				fi
				;;
			    *)
				MATCHED="0"
				;;
			esac
		fi
	    done # for LINE ...
    done # while true
    IFS=$IFS_PREV
}

display_menuitem()
{
    CHOICE=$1
    NAME=$2
    echo -e "${GOOD}**\t[${HILITE}${CHOICE}${GOOD}]${HILITE} ${NAME}${NORMAL}"
}

display_menu()
{
    STR_LENGTH=${#1}
    STR_LINE=$(while [ "$STR_LENGTH" -ne "0" ]; do printf "%s" "-"; let "STR_LENGTH-=1"; done; printf "\n")
    good ""
    hilite $1
    hilite $STR_LINE
    good ""
    good "You have the following options:"
    good ""
    # LINE and $2 can contain spaces
    for LINE in `cat $3 | grep "^$2:"`
	do
	    NAME=$(echo $LINE | cut -d':' -f2)
	    LINECHOICE=$(echo $LINE |cut -d':' -f3)
	    display_menuitem $NAME $LINECHOICE
	done
    good ""
    display_menuitem "0" "Return"
    display_menuitem "Q" "Exit"
    good ""
    getparam "Enter your choice:"
}

isdaemonrunning()
{
    DAEMON=$1
    PID=$(cat "${INSTALLDIR}/var/${DAEMON}.pid" 2>/dev/null)
    kill -0 $PID 2>/dev/null
}

# Returns 0 if server is running
# <> 0 if server is not running
check_server_is_running()
{
	SERVER_IS_RUNNING="1"
    CMD_OUT=$(ps ax | grep merakd | grep -v grep 2> /dev/null)
    if [ -n "$CMD_OUT" ]; then
    	SERVER_IS_RUNNING="0"
    fi
    CMD_OUT=$(ps ax | grep icewarpd | grep -v grep 2> /dev/null)
    if [ -n "$CMD_OUT" ]; then
    	SERVER_IS_RUNNING="0"
    fi
    return $SERVER_IS_RUNNING
}

run_cmd_log()
{
    # log command
    DATE_TIME=$(date "+%Y-%m-%d %H:%M:%S.%N")
    echo "${DATE_TIME}: \"$1\" \"$2\" \"$3\" \"$4\" \"$5\" \"$6\" \"$7\" \"$8\" \"$9\" \"${10}\" \"${11}\" \"${12}\" \"${13}\" \"${14}\" \"${15}\"" >> "$WIZARDCMDLOG"

    # run command, catch only stdout
    # because Merak uses stdout
    # pass parameters one by one, because parameter can contain space
    # example is domain description
    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    CMD_STDOUT=$("$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9" "${10}" "${11}" "${12}" "${13}" "${14}" "${15}")
    CMD_RET=$?

    # log command output
    DATE_TIME=$(date "+%Y-%m-%d %H:%M:%S.%N")
    echo "${DATE_TIME}: Command output:" >> "$WIZARDCMDLOG"

    DATE_TIME=$(date "+%Y-%m-%d %H:%M:%S.%N")
    echo "${DATE_TIME}: ${CMD_STDOUT}" >> "$WIZARDCMDLOG"

    # log command result
    DATE_TIME=$(date "+%Y-%m-%d %H:%M:%S.%N")
    echo "${DATE_TIME}: Result: ${CMD_RET}" >> "$WIZARDCMDLOG"

    return $CMD_RET
}

run_cmd_print_stdout_log()
{
    CMD_STDOUT=""
    CMD_RET="0"
    run_cmd_log $*
    echo "$CMD_STDOUT"
    return $CMD_RET
}

preconf
