#!/bin/bash
#
# IceWarp Server
# (c) IceWarp Ltd. 2008
#
#       All rights reserved.
#
# http://www.icewarp.com
#
# file: install_aspell_dict.sh - installer for aspell dictionary
# version: 1.0
#

# init working dir
pushd "$(pwd)" &> /dev/null
WORK_DIR=$(dirname "$0")
cd "$WORK_DIR" &> /dev/null
if [ $? -ne 0 ]
    then
	echo "Can not change directory to ${WORK_DIR}"
	exit 1
fi

# include functions
if [ -z "$FUNCTIONS_INCLUDED" ]
    then
	source ./inc/functions.sh
fi

# command line parameters test
if [ $# -ne 1 ]; then
    echo "Incorrect count of comamnd line parameters"
    echo ""
    echo "Usage:"
    echo "$0 aspell_dictionary.tar.bz2"
    echo ""
    exit -1
fi

ASPELL_DICT_FILE_NAME=$1
ASPELL_DICT_BASE_FILE_NAME=$(basename $1)

FILE_EXT_REGEX='\.tar\.bz2$'

# test if $1 has extension .tar.bz2
if [[ ! "$ASPELL_DICT_BASE_FILE_NAME" =~ $FILE_EXT_REGEX ]]; then
    echo "File name is not bz2 archive"
fi

# create temp directory
TEMP_DIR=$(mktemp -dt)
if [ $? -ne 0 ]; then
    echo "Cannot create temporary directory"
    exit -1
fi

DICT_DIR="$TEMP_DIR/`echo \"$ASPELL_DICT_BASE_FILE_NAME\" | sed \"s|$FILE_EXT_REGEX||\"`"

# extract archive into temporary directory
tar -xjf "$ASPELL_DICT_FILE_NAME" -C "$TEMP_DIR"
if [ $? -ne 0 ]; then
    echo "Cannot extract dictionary package"
    exit -1
fi

# change to extracted dictionary directory
cd "$DICT_DIR"
if [ $? -ne 0 ]; then
    echo "Can not change to dicrectory: ${DICT_DIR}"
    exit -1
fi

export PATH="${INSTALLDIR}/php/bin:${PATH}"
export LD_LIBRARY_PATH="${INSTALLDIR}/php/lib:${LD_LIBRARY_PATH}"
export ASPELL_PREFIX="${INSTALLDIR}/php"

# configure package
./configure
if [ $? -ne 0 ]; then
    echo "Error configuring aspell dictionary"
    exit -1
fi

make
if [ $? -ne 0 ]; then
    echo "Error making aspell dictionary"
    exit -1
fi

make install
if [ $? -ne 0 ]; then
    echo "Error installing aspell dictionary"
    exit -1
fi

# delete temporary directory
rm -rf "$TEMP_DIR"

# print info message
echo "----------"
echo "Aspell dictionary: ${ASPELL_DICT_FILE_NAME} installed OK."

# restore working dir
popd &> /dev/null
exit 0
