#!/bin/bash
#
# IceWarp Server uninstall script
# (c) IceWarp Ltd. 2008
#
#       All rights reserved.
#
# http://www.icewarp.com
#
# file: uninstall.sh - uninstall script
# version: 1.1
#

# init working dir
pushd "$(pwd)" &> /dev/null
WORK_DIR=$(dirname "$0")
cd "$WORK_DIR" &> /dev/null
if [ $? -ne 0 ]
    then
	echo "Can not change directory to ${WORK_DIR}"
	exit 1
fi

# include functions
if [ -z "$FUNCTIONS_INCLUDED" ]
    then
	source ./inc/functions.sh
fi

# start uninstall
uninstall_cancelled_quit()
{
    if [ -n "$1" ]
	then
	    echo "$1"
    fi
    echo "Uninstall cancelled."
    exit 1;
}

check_server_is_running
if [ $? -eq 0 ]
    then
	bad "IceWarp Server is running."
	bad "Please stop server before uninstalling."
	exit 1
fi

ask_with_confirmation "UNINSTALL IceWarp Server FROM ${INSTALLDIR}?" "N" "y"
if [ $? -ne 2 ]
    then
	uninstall_cancelled_quit
fi

echo "Uninstalling IceWarp Server..."

# load directories and files list
if [ -f "${INSTALLDIR}/scripts/setup/installed_dirs.txt" ]
    then
		INSTALLED_DIRS=$(cat "${INSTALLDIR}/scripts/setup/installed_dirs.txt")
    else
		uninstall_cancelled_quit "Can not find istalled directories list file."
fi
if [ -f "${INSTALLDIR}/scripts/setup/installed_files.txt" ]
    then
		INSTALLED_FILES=$(cat "${INSTALLDIR}/scripts/setup/installed_files.txt")
    else
		uninstall_cancelled_quit "Can not find istalled files list file."
fi

# change to install directory
cd "$INSTALLDIR" &> /dev/null
if [ $? -ne 0 ]
    then
	uninstall_cancelled_quit "Can not change directory to ${INSTALLDIR}"
fi

# remove all installed files
for LINE in $INSTALLED_FILES
do
    rm -f "${INSTALLDIR}/${LINE}"
done

# remove all installed directories
for LINE in $INSTALLED_DIRS
do
    rmdir -p "${INSTALLDIR}/${LINE}" &> /dev/null
done

# remove top directory
cd "${INSTALLDIR}/.."
rmdir "$INSTALLDIR"
if [ -d "$INSTALLDIR" ]
    then
	echo "Directory ${INSTALLDIR} was not removed."
fi

# unregister service for RHEL and ROOT
UUID=$(id -u)
OS_PLATFORM="UNKNOWN"
if [ -f "/etc/redhat-release" ]; then
	OS_PLATFORM="RHEL"
fi
if [ "$UUID" -eq 0 ] && [ "$OS_PLATFORM" == "RHEL" ]; then
	echo "Trying to remove 'icewarp' service from system..."
	chkconfig --del icewarp &> /dev/null
	rm -f "/etc/init.d/icewarp" &> /dev/null
fi

echo
echo "Uninstall done."

# restore working dir
popd &> /dev/null
exit 0
